package net.outlyer.plugins.sandboxing;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URI;

/**
 * java.io.Reader for plugins, tries to abstract the plugin location (file or jar)
 */
class PluginReader extends Reader {

    private final Reader readerImpl;

    PluginReader(final URI uri) throws IOException {
        super();
        assert null != uri;
        try {
            if (uri.getScheme().equals("file")) {
                readerImpl = new FileReader(new File(uri));
            }
            else {
                final String path = uri.getSchemeSpecificPart().split("!")[1];
                final InputStream is = getClass().getResourceAsStream(path);
                
                if (null == is) {
                    throw new IOException("Failed to get resource for " + uri);
                }
                
                readerImpl = new InputStreamReader(is);
            }
        }
        catch (final FileNotFoundException e) {
            throw new IOException("Can\'t read input " + uri);
        }
    }

    @Override
    public int read(char[] cbuf, int off, int len) throws IOException {
        return readerImpl.read(cbuf, off, len);
    }

    @Override
    public void close() throws IOException {
        readerImpl.close();
    }
}
